import numpy as np
import matplotlib.pyplot as plt
import itertools
from mpl_toolkits.mplot3d import Axes3D


def nSamples(n):
    return n * (n + 1) / 2


# samples should be an m x n matrix: each row is a sample, each column is a coupling value
nOps = 2  # number of operators being considered: important so that we know how to construct/interpret the coupling matrices

samples = np.array(
    [
        [1.0, 0.0, 0.0],
        [1.0, 2.0, 0.0],
        [1.0, 2.0, 50.0],
        [1.0, 2.0, -50.0],
        [1.0, -2.0, 50.0],
        [1.0, -2.0, -50.0],
    ]
)
# samples = {1.0, 0.0, 0.0}, {1.0, 2.0, 0.0}, {1.0, 2.0, 50.0}, {1.0, 2.0, -50.0}, {1.0, -2.0, 50.0}, {1.0, -2.0, -50.0},

# the n values of c_i that define the 'input' distributions: c_sm, c_i1...c_in
# in this input format, each row corresponds to the couplings for a given input sample
# ordering SM^2, SM*C1...SM*Cn, Ci*Cj...

print("n prediction required is  = " + str(nSamples(nOps + 1)))

# cInputAr is generated automatically from 'samples'
nTerms = 1 + (2 * nOps) + len(list(itertools.combinations(samples[0], 2))) - nOps

print("n terms is  = " + str(nTerms))

cInputAr = np.array([])

for sample in samples:
    couplings = np.array([])
    combs = list(itertools.combinations(sample, 2))
    # SM term
    couplings = np.append(couplings, sample[0] ** 2)
    #'SM--Oi' interference terms
    for comb in range(0, nOps):
        couplings = np.append(couplings, combs[comb][0] * combs[comb][1])
    #'squared' terms
    for ci in range(1, nOps + 1):
        couplings = np.append(couplings, sample[ci] ** 2)
    #'cross' terms
    for comb in range(nOps, len(combs)):
        couplings = np.append(couplings, combs[comb][0] * combs[comb][1])
    cInputAr = np.append(cInputAr, couplings, axis=0)

cInputAr = np.reshape(cInputAr, (nTerms, nTerms))
# cInputAr = np.array([[1.0, 0.0, 0.0, 0.0, 0.0, 0.0], [1.0, 2.0, 0.0, 4.0, 0.0, 0.0], [1.0, 2.0, 50.0, 4.0, 2500.0, 100.0],[1.0, 2.0, -50.0, 4.0, 2500.0, -100.0], [1.0, -2.0, 50.0, 4.0, 2500.0, -100.0], [1.0, -2.0, -50.0, 4.0, 2500.0, 100.0]])
cInputAr = np.transpose(cInputAr)
inWeightsAr = np.linalg.inv(cInputAr)
Tin = np.array([[20.46], [34.54], [158.9], [151.6], [135.6], [132.2]])
TinT = Tin.transpose()

# 'sanity check' example
ciAr = np.array([])
cjAr = np.array([])
# xs = np.array([])


def calcXS(ci, cj):
    ciTargetAr = np.array([[1.0], [ci], [cj], [ci ** 2], [cj ** 2], [ci * cj]])
    # ciTargetAr = np.array([])
    morphWeightsAr = inWeightsAr.dot(ciTargetAr)
    Tout = TinT.dot(morphWeightsAr)
    # print("ci  = " + str(ci) + ", cj = " + str(cj))
    return Tout


print("val prediction for ci = 3.65, cj = 17.12 = " + str(calcXS(3.65, 17.12)))
print("MadGraph prediction for ci = 3.65, cj = 17.12 = 66.55")

vCalcXS = np.vectorize(calcXS)

ciAr = np.arange(-8, 5, 0.5)
cjAr = np.arange(-40, 40, 0.5)
X, Y = np.meshgrid(ciAr, cjAr)

plt.figure()

xs = vCalcXS(X, Y)

print("xs shape = " + str(xs.shape))
cols = np.arange(0, 85, 1)
contour_filled = plt.contourf(ciAr, cjAr, xs, levels=cols)
cbar = plt.colorbar(contour_filled)
cbar.set_label(r"$\sigma_{t\bar{t}}$ [pb]", rotation=90)

plt.xlabel(r"$c_tG$")
plt.ylabel(r"$c^{3,8}_Qq$")

plt.show()

# valPtsX = [-3.31, 5.72]
# valPtsY = [11.17, 77.03]

# inPtsX = [0.0, 2.0, -2.0]
# inPtsY = [20.76, 34.38, 12.71]

# fig, ax = plt.subplots()
# ax.set_xlabel('ctg')
# ax.set_ylabel('cross section [pb]')

# plt.plot(x,y, label='morphing model')
# plt.plot(inPtsX, inPtsY, "b+", markersize=7, label='input predictions')
# plt.plot(valPtsX, valPtsY, "r+", markersize=7,label='validation predictions')

# legend = plt.legend(loc='upper center', shadow=True, fontsize='x-large')

# plt.show()



from asciimatics.effects import Cycle, Stars
from asciimatics.renderers import FigletText
from asciimatics.scene import Scene
from time import sleep


def deft_splash(screen):
    effects = [
        Cycle(screen, FigletText("dEFT", font="big"), int(screen.height / 2 - 8)),
        Cycle(
            screen,
            FigletText("A differential Effective Field Theory tool", font="small"),
            int(screen.height / 2 + 3),
        ),
        Cycle(
            screen,
            FigletText("press q to run job!", font="small"),
            int(screen.height / 1.5),
        ),
        Stars(screen, 900),
    ]
    screen.play([Scene(effects, 900)])
    ev = screen.get_key()
    if ev in (ord("Q"), ord("q")):
        return



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

import functools
import random
import sys
import time

import progressbar

examples = []


def example(fn):
    """Wrap the examples so they generate readable output"""

    @functools.wraps(fn)
    def wrapped():
        try:
            sys.stdout.write("Running: %s\n" % fn.__name__)
            fn()
            sys.stdout.write("\n")
        except KeyboardInterrupt:
            sys.stdout.write("\nSkipping example.\n\n")
            # Sleep a bit to make killing the script easier
            time.sleep(0.2)

    examples.append(wrapped)
    return wrapped


@example
def shortcut_example():
    for i in progressbar.progressbar(range(10)):
        time.sleep(0.1)


@example
def prefixed_shortcut_example():
    for i in progressbar.progressbar(range(10), prefix="Hi: "):
        time.sleep(0.1)


@example
def templated_shortcut_example():
    for i in progressbar.progressbar(range(10), suffix="{seconds_elapsed:.1}"):
        time.sleep(0.1)


@example
def with_example_stdout_redirection():
    with progressbar.ProgressBar(max_value=10, redirect_stdout=True) as p:
        for i in range(10):
            if i % 3 == 0:
                print("Some print statement %i" % i)
            # do something
            p.update(i)
            time.sleep(0.1)


@example
def basic_widget_example():
    widgets = [progressbar.Percentage(), progressbar.Bar()]
    bar = progressbar.ProgressBar(widgets=widgets, max_value=10).start()
    for i in range(10):
        # do something
        time.sleep(0.1)
        bar.update(i + 1)
    bar.finish()


@example
def file_transfer_example():
    widgets = [
        "Test: ",
        progressbar.Percentage(),
        " ",
        progressbar.Bar(marker=progressbar.RotatingMarker()),
        " ",
        progressbar.ETA(),
        " ",
        progressbar.FileTransferSpeed(),
    ]
    bar = progressbar.ProgressBar(widgets=widgets, max_value=1000).start()
    for i in range(100):
        # do something
        bar.update(10 * i + 1)
    bar.finish()


@example
def custom_file_transfer_example():
    class CrazyFileTransferSpeed(progressbar.FileTransferSpeed):

        "It's bigger between 45 and 80 percent"

        def update(self, bar):
            if 45 < bar.percentage() < 80:
                return "Bigger Now " + progressbar.FileTransferSpeed.update(self, bar)
            else:
                return progressbar.FileTransferSpeed.update(self, bar)

    widgets = [
        CrazyFileTransferSpeed(),
        " <<<",
        progressbar.Bar(),
        ">>> ",
        progressbar.Percentage(),
        " ",
        progressbar.ETA(),
    ]
    bar = progressbar.ProgressBar(widgets=widgets, max_value=1000)
    # maybe do something
    bar.start()
    for i in range(200):
        # do something
        bar.update(5 * i + 1)
    bar.finish()


@example
def double_bar_example():
    widgets = [
        progressbar.Bar(">"),
        " ",
        progressbar.ETA(),
        " ",
        progressbar.ReverseBar("<"),
    ]
    bar = progressbar.ProgressBar(widgets=widgets, max_value=1000).start()
    for i in range(100):
        # do something
        bar.update(10 * i + 1)
        time.sleep(0.01)
    bar.finish()


@example
def basic_file_transfer():
    widgets = [
        "Test: ",
        progressbar.Percentage(),
        " ",
        progressbar.Bar(marker="0", left="[", right="]"),
        " ",
        progressbar.ETA(),
        " ",
        progressbar.FileTransferSpeed(),
    ]
    bar = progressbar.ProgressBar(widgets=widgets, max_value=500)
    bar.start()
    # Go beyond the max_value
    for i in range(100, 501, 50):
        time.sleep(0.1)
        bar.update(i)
    bar.finish()


@example
def simple_progress():
    bar = progressbar.ProgressBar(
        widgets=[progressbar.SimpleProgress()],
        max_value=17,
    ).start()
    for i in range(17):
        time.sleep(0.1)
        bar.update(i + 1)
    bar.finish()


@example
def basic_progress():
    bar = progressbar.ProgressBar().start()
    for i in range(10):
        time.sleep(0.1)
        bar.update(i + 1)
    bar.finish()


@example
def progress_with_automatic_max():
    # Progressbar can guess max_value automatically.
    bar = progressbar.ProgressBar()
    for i in bar(range(8)):
        time.sleep(0.1)


@example
def progress_with_unavailable_max():
    # Progressbar can't guess max_value.
    bar = progressbar.ProgressBar(max_value=8)
    for i in bar((i for i in range(8))):
        time.sleep(0.1)


@example
def animated_marker():
    bar = progressbar.ProgressBar(widgets=["Working: ", progressbar.AnimatedMarker()])
    for i in bar((i for i in range(5))):
        time.sleep(0.1)


@example
def counter_and_timer():
    widgets = [
        "Processed: ",
        progressbar.Counter("Counter: %(value)05d"),
        " lines (",
        progressbar.Timer(),
        ")",
    ]
    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar((i for i in range(15))):
        time.sleep(0.1)


@example
def format_label():
    widgets = [progressbar.FormatLabel("Processed: %(value)d lines (in: %(elapsed)s)")]
    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar((i for i in range(15))):
        time.sleep(0.1)


@example
def animated_balloons():
    widgets = ["Balloon: ", progressbar.AnimatedMarker(markers=".oO@* ")]
    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar((i for i in range(24))):
        time.sleep(0.1)


@example
def animated_arrows():
    # You may need python 3.x to see this correctly
    try:
        widgets = ["Arrows: ", progressbar.AnimatedMarker(markers="←↖↑↗→↘↓↙")]
        bar = progressbar.ProgressBar(widgets=widgets)
        for i in bar((i for i in range(24))):
            time.sleep(0.1)
    except UnicodeError:
        sys.stdout.write("Unicode error: skipping example")


@example
def animated_filled_arrows():
    # You may need python 3.x to see this correctly
    try:
        widgets = ["Arrows: ", progressbar.AnimatedMarker(markers="◢◣◤◥")]
        bar = progressbar.ProgressBar(widgets=widgets)
        for i in bar((i for i in range(24))):
            time.sleep(0.1)
    except UnicodeError:
        sys.stdout.write("Unicode error: skipping example")


@example
def animated_wheels():
    # You may need python 3.x to see this correctly
    try:
        widgets = ["Wheels: ", progressbar.AnimatedMarker(markers="◐◓◑◒")]
        bar = progressbar.ProgressBar(widgets=widgets)
        for i in bar((i for i in range(24))):
            time.sleep(0.1)
    except UnicodeError:
        sys.stdout.write("Unicode error: skipping example")


@example
def format_label_bouncer():
    widgets = [
        progressbar.FormatLabel("Bouncer: value %(value)d - "),
        progressbar.BouncingBar(),
    ]
    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar((i for i in range(100))):
        time.sleep(0.01)


@example
def format_label_rotating_bouncer():
    widgets = [
        progressbar.FormatLabel("Animated Bouncer: value %(value)d - "),
        progressbar.BouncingBar(marker=progressbar.RotatingMarker()),
    ]

    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar((i for i in range(18))):
        time.sleep(0.1)


@example
def with_right_justify():
    with progressbar.ProgressBar(
        max_value=10, term_width=20, left_justify=False
    ) as progress:
        assert progress.term_width is not None
        for i in range(10):
            progress.update(i)


@example
def exceeding_maximum():
    with progressbar.ProgressBar(max_value=1) as progress:
        try:
            progress.update(2)
        except ValueError:
            pass


@example
def reaching_maximum():
    progress = progressbar.ProgressBar(max_value=1)
    try:
        progress.update(1)
    except RuntimeError:
        pass


@example
def stdout_redirection():
    with progressbar.ProgressBar(redirect_stdout=True) as progress:
        print("", file=sys.stdout)
        progress.update(0)


@example
def stderr_redirection():
    with progressbar.ProgressBar(redirect_stderr=True) as progress:
        print("", file=sys.stderr)
        progress.update(0)


@example
def negative_maximum():
    try:
        with progressbar.ProgressBar(max_value=-1) as progress:
            progress.start()
    except ValueError:
        pass


@example
def rotating_bouncing_marker():
    widgets = [progressbar.BouncingBar(marker=progressbar.RotatingMarker())]
    with progressbar.ProgressBar(
        widgets=widgets, max_value=20, term_width=10
    ) as progress:
        for i in range(20):
            time.sleep(0.1)
            progress.update(i)

    widgets = [
        progressbar.BouncingBar(marker=progressbar.RotatingMarker(), fill_left=False)
    ]
    with progressbar.ProgressBar(
        widgets=widgets, max_value=20, term_width=10
    ) as progress:
        for i in range(20):
            time.sleep(0.1)
            progress.update(i)


@example
def incrementing_bar():
    bar = progressbar.ProgressBar(
        widgets=[
            progressbar.Percentage(),
            progressbar.Bar(),
        ],
        max_value=10,
    ).start()
    for i in range(10):
        # do something
        time.sleep(0.1)
        bar += 1
    bar.finish()


@example
def increment_bar_with_output_redirection():
    widgets = [
        "Test: ",
        progressbar.Percentage(),
        " ",
        progressbar.Bar(marker=progressbar.RotatingMarker()),
        " ",
        progressbar.ETA(),
        " ",
        progressbar.FileTransferSpeed(),
    ]
    bar = progressbar.ProgressBar(
        widgets=widgets, max_value=1000, redirect_stdout=True
    ).start()
    for i in range(100):
        # do something
        time.sleep(0.01)
        bar += 10
        print("Got", i)
    bar.finish()


@example
def eta_types_demonstration():
    widgets = [
        progressbar.Percentage(),
        " ETA: ",
        progressbar.ETA(),
        " Adaptive ETA: ",
        progressbar.AdaptiveETA(),
        " Absolute ETA: ",
        progressbar.AbsoluteETA(),
        " Adaptive Transfer Speed: ",
        progressbar.AdaptiveTransferSpeed(),
        " ",
        progressbar.Bar(),
    ]
    bar = progressbar.ProgressBar(widgets=widgets, max_value=500)
    bar.start()
    for i in range(500):
        if i < 100:
            time.sleep(0.02)
        elif i > 400:
            time.sleep(0.1)
        else:
            time.sleep(0.01)
        bar.update(i + 1)
    bar.finish()


@example
def adaptive_eta_without_value_change():
    # Testing progressbar.AdaptiveETA when the value doesn't actually change
    bar = progressbar.ProgressBar(
        widgets=[
            progressbar.AdaptiveETA(),
            progressbar.AdaptiveTransferSpeed(),
        ],
        max_value=2,
        poll_interval=0.0001,
    )
    bar.start()
    for i in range(100):
        bar.update(1)
        time.sleep(0.1)
    bar.finish()


@example
def iterator_with_max_value():
    # Testing using progressbar as an iterator with a max value
    bar = progressbar.ProgressBar()

    for n in bar(iter(range(100)), 100):
        # iter range is a way to get an iterator in both python 2 and 3
        pass


@example
def eta():
    widgets = [
        "Test: ",
        progressbar.Percentage(),
        " | ETA: ",
        progressbar.ETA(),
        " | AbsoluteETA: ",
        progressbar.AbsoluteETA(),
        " | AdaptiveETA: ",
        progressbar.AdaptiveETA(),
    ]
    bar = progressbar.ProgressBar(widgets=widgets, max_value=50).start()
    for i in range(50):
        time.sleep(0.1)
        bar.update(i + 1)
    bar.finish()


@example
def dynamic_message():
    # Use progressbar.DynamicMessage to keep track of some parameter(s) during
    # your calculations
    widgets = [
        progressbar.Percentage(),
        progressbar.Bar(),
        progressbar.DynamicMessage("loss"),
    ]
    with progressbar.ProgressBar(max_value=100, widgets=widgets) as bar:
        min_so_far = 1
        for i in range(100):
            val = random.random()
            if val < min_so_far:
                min_so_far = val
            bar.update(i, loss=min_so_far)


@example
def format_custom_text():
    format_custom_text = progressbar.FormatCustomText(
        "Spam: %(spam).1f kg, eggs: %(eggs)d",
        dict(
            spam=0.25,
            eggs=3,
        ),
    )

    bar = progressbar.ProgressBar(
        widgets=[
            format_custom_text,
            " :: ",
            progressbar.Percentage(),
        ]
    )
    for i in bar(range(25)):
        format_custom_text.update_mapping(eggs=i * 2)
        time.sleep(0.1)


@example
def simple_api_example():
    bar = progressbar.ProgressBar(widget_kwargs=dict(fill="█"))
    for i in bar(range(200)):
        time.sleep(0.02)


@example
def ETA_on_generators():
    def gen():
        for x in range(200):
            yield None

    widgets = [
        progressbar.AdaptiveETA(),
        " ",
        progressbar.ETA(),
        " ",
        progressbar.Timer(),
    ]

    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar(gen()):
        time.sleep(0.02)


@example
def percentage_on_generators():
    def gen():
        for x in range(200):
            yield None

    widgets = [
        progressbar.Counter(),
        " ",
        progressbar.Percentage(),
        " ",
        progressbar.SimpleProgress(),
        " ",
    ]

    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar(gen()):
        time.sleep(0.02)


def test(*tests):
    for example in examples:
        if not tests or example.__name__ in tests:
            example()
        else:
            print("Skipping", example.__name__)


if __name__ == "__main__":
    try:
        test(*sys.argv[1:])
    except KeyboardInterrupt:
        sys.stdout("\nQuitting examples.\n")



import numpy as np
import matplotlib.pyplot as plt

# the n values of c_i that define the 'input' distributions: c_sm, c_i1...c_in
# there are Nops columns.
# first column is the SM coupling squared values for each of
# the input samples,
# second is the squared value of the second term of the cross section expansion
# and so on for the third and subsequent solumns
# The convention here is that after the SM term, the squared term for the
# c_i comes next, then the ci-SM interference term, before continuing on
# for c_i+1 etc.

cInputAr = np.array([[1.0, 1.0, 1.0], [0.0, 4.0, 4.0], [0.0, 2.0, -2.0]])
inWeightsAr = np.linalg.inv(cInputAr)

Tin = np.array([[20.76], [34.48], [12.71]])
TinT = Tin.transpose()

# 'sanity check' example
y = np.array([])
x = np.array([])

for target in range(-10, 10):
    ciTargetAr = np.array([[1], [target ** 2.0], [target]])
    morphWeightsAr = inWeightsAr.dot(ciTargetAr)
    Tout = TinT.dot(morphWeightsAr)
    print("target = " + str(target) + " tout " + str(Tout))
    x = np.append(x, target)
    y = np.append(y, Tout)


valPtsX = [-3.31, 5.72]
valPtsY = [11.17, 77.03]


inPtsX = [0.0, 2.0, -2.0]
inPtsY = [20.76, 34.38, 12.71]

fig, ax = plt.subplots()
ax.set_xlabel("ctg")
ax.set_ylabel("cross section [pb]")

plt.plot(x, y, label="morphing model")
plt.plot(inPtsX, inPtsY, "b+", markersize=7, label="input predictions")
plt.plot(valPtsX, valPtsY, "r+", markersize=7, label="validation predictions")

legend = plt.legend(loc="upper center", shadow=True, fontsize="x-large")

plt.show()



import re as _re
from pathlib import Path as _Path
import numpy as _np

# nBins = 5
# nOps = 6
# obs = "ptZ"

# procDirName = "twz_train_6ops_lo"
# procFileStem = "TWZ_6ops_train_"


def convert_hwu_to_numpy(filename: _Path, num_of_hist: int = 1) -> (_np.ndarray, _np.ndarray):
    '''
    Convert a HwU (histogram with uncertainties) file from MadGraph into a JSON file which is compatible with dEFT.
    Only parses the first histogram in the file by default.
    '''
    with open(filename, 'r') as hwu_f:
        bin_bounds = _np.empty(0)
        central_values = _np.empty(0)
        for i in range(num_of_hist):
            line = next(hwu_f)
            while (match := _re.search(r"<histogram> (\d+) ", line)) is None:
                line = next(hwu_f)
                if line is None:
                    raise RuntimeError(f"No histogram found in HwU file: {filename}")
            bins = int(match.group(1))

            line = next(hwu_f)
            for i in range(bins):
                # The 2-index in HwU are central value of bin
                values = line.strip().split()
                central_values = _np.append(central_values, float(values[2]))
                bin_bounds = _np.append(bin_bounds, float(values[0]))
                line = next(hwu_f)

    bin_bounds = _np.append(bin_bounds, float(values[1]))

    assert(len(bin_bounds) > len(central_values))
    return bin_bounds, central_values

    # startRun = 1
    # endRun = 150


    # read = "false"
    # central_values = []
    # final_array = []

    # for run in range(startRun, (endRun + 1)):
        # central_values = []
        # if run < 10:
            # run_string = "run_0" + str(run) + "_LO/MADatNLO.HwU"
        # else:
            # run_string = "run_" + str(run) + "_LO/MADatNLO.HwU"

        # file_string = hwufile + run_string
        # with open(file_string, "r") as f:
            # for line in f:
                # if obs in line:
                    # read = "true"
                # if read == "true":
                    # if len(line.split("   ")) == 15:
                        # # print("LINE " + str(line))
                        # # bin_width = float(line.split("\t")[1]) - float(line.split("\t")[0])
                        # central_values.append(float(line.split("   ")[2]))
                        # # print(line.split("   ")[2][1:]  )
                # if "<\histogram>" in line:
                    # read = "false"
        # print(str(central_values) + ",")
        # final_array = np.append(final_array, central_values)
    # final_array.reshape(((endRun - startRun) + 1), nBins)
    # return final_array


# preds = convert("../analyses/twz_train_6ops_lo/Events/")
# print(repr(preds.reshape(((endRun - startRun) + 1), nBins)))


# def convertRun(runfile):
    # # print "tools/yoda2array: converting yodafile to numpy array"
    # read = "false"
    # central_values = np.array([])
    # final_array = []

    # # print("converting " + str(runfile))

    # central_values = []

    # with open(runfile, "r") as f:
        # for line in f:
            # if "fixed_order=ON" in line:
                # read = "true"
                # central_values = np.append(central_values, float(1.00000001))
            # if read == "true":
                # if len(line.split(" ")) > 3:
                    # central_values = np.append(
                        # central_values, float(line.split(" ")[3])
                    # )
                    # # central_values.append(float(line.split(" ")[3]) )
            # if "#launch /tmp/" + procDirName + "/" in line:
                # read = "false"
        # # print(central_values)
        # # central_values.reshape(endRun, nOps)
    # return central_values


# totalPreds = np.array([])

# for p in range(0, 5):
    # fileName = procFileStem + str(p) + ".txt"
    # preds = convertRun(fileName)
    # totalPreds = np.concatenate((totalPreds, preds), axis=None)

# totalPreds = totalPreds.reshape(((endRun - startRun) + 1), (nOps + 1))

# print("shap totalPreds = " + str(totalPreds.shape))

# np.set_printoptions(threshold=sys.maxsize)
# print(np.array2string(totalPreds, separator=','))

# print(repr(totalPreds.reshape(endRun,(nOps+1))))

# for pred in totalPreds:
#    print(str(repr(pred)) + ",")



from tools.predBuilder import predBuilder
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

import numpy as np

testSamples = [
    [1.0, -49.0, -46.0],
    [1.0, -41.0, 45.0],
    [1.0, -43.0, 40.0],
    [1.0, 49.0, -50.0],
    [1.0, 10.0, -46.0],
    [1.0, -49.0, 50.0],
    [1.0, 10.0, -11.0],
    [1.0, 20.0, -25.0],
    [1.0, 20.0, -20.0],
    [1.0, -10.0, -10.0],
    [1.0, 49.0, -48.0],
    [1.0, -200.0, -200.0],
]
testPreds = np.asarray(
    [
        [278],
        [169.2],
        [156.2],
        [192.2],
        [105.4],
        [211.9],
        [26.48],
        [52.67],
        [46.06],
        [29.72],
        [183.2],
        [4738],
    ]
)


prior_limits = {"cQq83": [-50.0, 25.0], "cQq81": [-40.0, 40.0]}

# preds = np.asarray( [[189.4], [182.0], [302], [258.6], [184.2], [290.1]])
preds = np.asarray([[189.7], [180.5], [299.8], [258.7], [183.4], [20.46]])

samples = [
    [1.0, -47.0, 46.0],
    [1.0, -50.0, 40.0],
    [1.0, 47.0, 47.0],
    [1.0, -48.0, -44.0],
    [1.0, -47.0, 44.0],
    [1.0, 0.0, 0.0],
]

pb = predBuilder()
pb.init(len(prior_limits), samples, preds)

vMakePred = np.vectorize(pb.makePred)

ciAr = np.arange(-100, 100, 5.0)
cjAr = np.arange(-100, 100, 5.0)


ciP = []
cjP = []
xs = []

# z = np.array([pb.makePred(([ci, cj])) for ci in ciAr for cj in cjAr])

z = np.array([])

for ci in ciAr:
    for cj in cjAr:
        z = np.append(z, pb.makePred([ci, cj]))
        print(
            "ci = "
            + str(ci)
            + " cj = "
            + str(cj)
            + " z = "
            + str(pb.makePred([ci, cj]))
        )


z = z.reshape(len(cjAr), len(ciAr))

print("z shape = " + str(z.shape))

print("z max = " + str(np.amax(z)))
print("z min = " + str(np.amin(z)))

cols = np.arange(np.amin(z), np.amax(z), 1)
contour_filled = plt.contourf(cjAr, ciAr, z, levels=cols)

contour = plt.contour(cjAr, ciAr, z, levels=[0.0])

cbar = plt.colorbar(contour_filled)
cbar.set_label(r"$\sigma_{t\bar{t}}$ [pb]", rotation=90)

plt.xlabel(r"$cQq83$")
plt.ylabel(r"$cQq81$")

plt.show()


# Z = z.reshape(len(ciAr), len(ciAr))
# plt.imshow(Z, interpolation='bilinear')
# plt.savefig("test.png")


# x = data[:, 0]
# y = data[:, 1]
# z = data[:, 2]

# for ci in ciAr:
#    for cj in cjAr:
#        cPoint = ([ci, cj])
##        ciP = np.append(ciP,ci)
#        cjP = np.append(cjP,cj)
#        xs = np.append(xs,pb.makePred(cPoint) )
# print(str(ci) + ", " + str(cj) + ", " + str(pb.makePred(cPoint)) )

# fig = plt.figure(figsize=(8, 6))
# ax = fig.add_subplot(111, projection="3d")
# ax.plot_trisurf(ciP,cjP,xs)

# cols = np.arange(0, 85, 1)
# contour_filled = plt.contourf(ciP, cjP, xs, levels=cols)
# cbar = plt.colorbar(contour_filled)
# cbar.set_label(r"$\sigma_{t\bar{t}}$ [pb]", rotation=90)

# plt.xlabel(r"$c_tG$")
# plt.ylabel(r"$c^{3,8}_Qq$")

# plt.show()



import numpy as np
from array import *
import math
import sys
import matplotlib.pyplot as pl
from matplotlib import gridspec
from mpl_toolkits.axes_grid1 import make_axes_locatable

nBins = 7
nOps = 7
obs = "ptZ"

procDirName = "tzq_train"
procFileStem = "TZQ_7ops_train_"


def plotHWU(hwufile):
    central_values = []
    file_string = hwufile

    lumi = 3000000  # pb^{-1}
    eff = 0.4
    BR = 0.0115

    scale = lumi * eff * BR

    with open(file_string, "r") as f:
        for line in f:
            if obs in line:
                read = "true"
            if read == "true":
                print("N " + str(len(line.split("   "))))

                if len(line.split("   ")) == 11:
                    print("LINE " + str(line))
                    # bin_width = float(line.split("\t")[1]) - float(line.split("\t")[0])
                    central_values.append(float(line.split("   ")[2]))
                    # print(line.split("   ")[2][1:]  )
                if "<\histogram>" in line:
                    read = "false"
    print(str(central_values) + ",")

    central_values = [0.05300887, 0.03509872, 0.0088587, 0.00106592, 0.00062776]
    dsig = [(x * 10.0) for x in central_values]

    final_values = [x * scale for x in central_values]
    stat_unc = np.sqrt(final_values)
    rel_stat_unc = stat_unc / final_values

    xvals = np.linspace(50, 450, 5)

    print("xvals " + str(len(xvals)))
    print("final_values " + str(len(final_values)))
    print("sum of central values " + str(sum(central_values)))
    print("rel stat unc  " + str(rel_stat_unc))

    # fill special arrays for error bands
    bin_edges = np.linspace(0, 500, 6)
    bin_edges_for_errors = np.array([])
    final_values_for_band = np.array([])

    stat_unc_for_band = np.array([])
    stat_unc_for_ratio = np.array([])

    for bin in range(0, len(xvals)):
        bin_edges_for_errors = np.append(
            xvals[len(xvals) - bin - 1] + 50.0, bin_edges_for_errors
        )
        bin_edges_for_errors = np.append(
            xvals[len(xvals) - bin - 1] - 50.0, bin_edges_for_errors
        )
        final_values_for_band = np.append(
            final_values[len(xvals) - bin - 1], final_values_for_band
        )
        final_values_for_band = np.append(
            final_values[len(xvals) - bin - 1], final_values_for_band
        )
        stat_unc_for_band = np.append(stat_unc[len(xvals) - bin - 1], stat_unc_for_band)
        stat_unc_for_band = np.append(stat_unc[len(xvals) - bin - 1], stat_unc_for_band)
        stat_unc_for_ratio = np.append(
            ((stat_unc[len(xvals) - bin - 1]) / (final_values[len(xvals) - bin - 1]))
            * 100.0,
            stat_unc_for_ratio,
        )
        stat_unc_for_ratio = np.append(
            ((stat_unc[len(xvals) - bin - 1]) / (final_values[len(xvals) - bin - 1]))
            * 100.0,
            stat_unc_for_ratio,
        )

    print("befe = " + str(bin_edges_for_errors))
    print("fvefe = " + str(final_values_for_band))
    print("uefe = " + str(stat_unc_for_band))
    zeros_for_ratio = np.zeros(len(xvals) * 2)

    fig = pl.figure()
    spec = gridspec.GridSpec(ncols=1, nrows=2, height_ratios=[3, 1], hspace=0.0)
    ax0 = fig.add_subplot(spec[0])
    ax0.set_ylabel(r"$N_{tWZ}$", fontsize=16)
    ax0.yaxis.set_label_coords(-0.075, 0.83)
    ax0.errorbar(xvals, final_values, fmt="", ls="none", xerr=50.0)
    ax0.fill_between(
        bin_edges_for_errors,
        final_values_for_band - stat_unc_for_band,
        final_values_for_band + stat_unc_for_band,
        facecolor="#F0F8FF",
        alpha=1.0,
        edgecolor="none",
    )

    # ax1 = ax0.twinx()
    # ax1.set_ylabel(r'$\frac{d \sigma}{d p^{Z}_{T}}\; [fb/GeV]$', fontsize = 14)
    # ax1.yaxis.set_label_coords(1.06, 0.79)
    # ax1.tick_params(axis='y')
    # ax1.errorbar(xvals, dsig, fmt="",  ls='none', alpha=0.0, xerr=50.0)

    # ratio plot
    ax2 = fig.add_subplot(spec[1])
    ax2.errorbar(
        xvals, np.zeros(len(xvals)), fmt="", ls="none", xerr=50.0, label=r"$N_{tWZ}$"
    )
    ax2.fill_between(
        bin_edges_for_errors,
        zeros_for_ratio - stat_unc_for_ratio,
        zeros_for_ratio + stat_unc_for_ratio,
        facecolor="#F0F8FF",
        alpha=1.0,
        edgecolor="none",
    )

    ax2.xaxis.set_label_coords(0.85, -0.25)
    labelx = ax2.set_xlabel(r"$p^{Z}_{T} \;[GeV]$", fontsize=16)
    labely = ax2.set_ylabel(r"rel. unc. [%]", fontsize=10)

    fig.savefig("sm.png")
    pl.close()

    return rel_stat_unc * central_values


stat_unc = plotHWU("sm_pred")


def plotDataCov(stat_unc):

    cov_mat = np.diag(stat_unc ** 2)
    diag = np.array([])

    cov1 = 0.05

    for bin in range(1, len(stat_unc)):
        diag = np.append(diag, ((stat_unc[bin]) * (stat_unc[bin - 1]) * cov1))

    cov_mat = cov_mat + np.diag(diag, -1) + np.diag(diag, 1)

    print(repr(cov_mat))

    pl.figure()
    im = pl.matshow(cov_mat, cmap=pl.cm.Blues, extent=[0.0, 500.0, 0.0, 500.0])
    pl.xlabel(r"$p^{Z}_{T} \;[GeV]$")
    pl.ylabel(r"$p^{Z}_{T} \;[GeV]$")

    ax = pl.gca()

    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="5%", pad=0.05)
    pl.colorbar(im, cax=cax)
    ax.tick_params(axis="x", bottom=True, top=False, labelbottom=True, labeltop=False)
    ax.xaxis.set_label_coords(0.85, -0.08)
    ax.yaxis.set_label_coords(-0.12, 0.9)

    pl.savefig("data_cov.png")
    pl.close()

    return


plotDataCov(stat_unc)



preds = []

with open("run_twz_8out", "r") as a_file:
    for line in a_file:
        if "Cross-section" in line:
            token = line.split(" ")
            pred = []
            pred.append(float(token[9]))
            preds.append(pred)

print("predictions = ")
print(preds)



import numpy as np
import matplotlib.pyplot as pl

from .ConfigReader import ConfigReader
from .PredictionBuilder import PredictionBuilder


class ModelValidator:
    """
    Allows for a model to be validated against dataset with known operator paramters (like MC signal). The model must be defined using `PredictionBuilder` class.
    """
    def __init__(self, pb: PredictionBuilder):
        self.nOps = pb.nOps
        self.pb = pb

    def validate(self, config_test: ConfigReader):
        test_samples = config_test.params["config"]["model"]["samples"]
        test_preds = np.asarray(config_test.params["config"]["model"]["predictions"])
        assert (
            self.nOps != len(config_test.params["config"]["model"]["prior_limits"]),
            "Operator mismatch. Make sure the supplied ConfigReader has the correct number of operators for the model",
        )

        print("##############################################################")
        print("###############  VALIDATION OF MORPHING MODEL  ###############")
        print("##############################################################")

        model_preds = np.fromiter(
            (self.pb.make_prediction(sample[1:]) for sample in test_samples),
            float,
            len(test_samples),
        )
        assert(len(model_preds[0]) == len(test_preds[0]))

        diff_preds = np.square(model_preds - test_preds).sum(axis=1)



""" module to convert yoda histograms in yoda files into numpy arrays
    so that the user can directly suppply the yoda file as input into a dEFT
    fit
"""
import numpy as np

# from ROOT import TFile, TH1D
from array import *
import math


def convert(yodafile, histname):
    # print "tools/yoda2array: converting yodafile to numpy array"
    ignore_strings = ["# ID", "# xlow", "Total   ", "Underflow", "Overflow"]
    read = "false"
    central_values = []

    # print "converting " + str(yodafile)
    with open(yodafile, "r") as f:
        for line in f:
            if ("BEGIN" in line) & (histname in line):
                read = "true"
            if read == "true":
                if (len(line.split("\t")) == 7) & (
                    line.split("\t")[0] not in ignore_strings
                ):
                    # print "LINE " + str(line)
                    bin_width = float(line.split("\t")[1]) - float(line.split("\t")[0])
                    central_values.append(float(line.split("\t")[2]) / bin_width)
            if "END" in line:
                read = "false"
    return np.asarray(central_values)


"""
def convert2root(yodafile, histname):
    #print "tools/yoda2array: converting yodafile to root file"
    ignore_strings = ["# ID", "# xlow", "Total   ", "Underflow", "Overflow"]
    read = "false"
    central_values = []
    uncertainties = []
    bin_edges = []
    last_line = ""
    n_bin_line = 0

    with open(yodafile, 'r') as f:
        for line in f:
            #print "RAW-LINE = " + line  + " histname  "+ str(histname)
            if ( ("BEGIN" in line) & (histname in line)):
                read = "true"
            if (read == "true"):
                if ((len(line.split("\t")) == 7) & (line.split("\t")[0] not in ignore_strings) ):
                    n_bin_line = n_bin_line + 1
                    if(n_bin_line) == 1:
                         bin_edges.append(float(line.split("\t")[0]))
                         bin_edges.append(float(line.split("\t")[1]))
                    else:
                         bin_edges.append(float(line.split("\t")[1]))
                    bin_width = float(line.split("\t")[1]) - float(line.split("\t")[0])
                    central_values.append(float(line.split("\t")[2]) / bin_width )
                    uncertainties.append(float(math.sqrt(float(line.split("\t")[3]))) / bin_width )

            if ("END" in line):
                read = "false"

        bin_edges = sorted(bin_edges)
        edgesArray = array('d',bin_edges)
        #print "edges = " + str(edgesArray) 
        histo = TH1D("", "", len(bin_edges)-1, edgesArray)
        # fill histo
        for bin in range(0, len(central_values)):
            histo.SetBinContent(bin+1, central_values[bin])
            histo.SetBinError(bin+1, uncertainties[bin])
            #print "error bar is " + str(uncertainties[bin])
    return  histo
"""



from ROOT import *
import ROOT

"""
import ROOT
f = TFile("root_inputs/nlo_ctG_p2_lhe_nom_pt_nom_xxx.root")
h = f.Get("TTbarSpinDensityMatrix/snake_spinCorr_part_cut_0")

sigma_CtG_0_nlo = 687.14
sigma_CtG_neg2_nlo = 404.48
sigma_CtG_pos2_nlo = 1270.204
acceptance = 1.0
branching_ratio = 0.046
k_factor = 1.21
n_bases =22.0

fid_xsec = (sigma_CtG_pos2_nlo*k_factor*acceptance*n_bases)
integral = h.Integral()
h.Scale(fid_xsec/integral)

print "[",
for bin in range(0,h.GetNbinsX()):
    #print str(    h.GetBinCenter(bin+1)  -  ( (h.GetBinWidth(bin+1))/2.0)  ) + ",",
    print str(    h.GetBinContent(bin+1)    ) + ",",
print "]"


f2 = TFile("root_inputs/unfolded_data_181109.root")

h2 = f2.Get("TTbarSpinDensityMatrix/snake_spinCorr_absolute_data_b")

#print "[",
#for bin in range(0,h2.GetNbinsX()):
#    print str(    h2.GetBinContent(bin+1)  / (h2.GetBinWidth(bin+1))  ) + ",",
#print "]"

f3 = TFile("root_inputs/covmat_181109_Systematics_AllVars_final.root")
cov = f3.Get("TotalStatSystCovMatrix_AllVar_rebinnedB")

#print "[",
#for bini in range(0,cov.GetNbinsX()):
# print "[",
#for binj in range(0,cov.GetNbinsY()):
#    cov_ij = cov.GetBinContent(bini+1, binj+1)
#    print str(cov_ij) + ",",
#print "]"
#print "]"

"""


# TOP-17-014
f = TFile("/Users/keaveney/EFT_Fitter/EFT_Fitter/files/CtG_2_nominal_v10.root")
h = f.Get("CMS_dilepton_diff/ll_delphi_abs")

# for bin in range(0,h.GetNbinsX()):
# h.SetBinContent(bin+1, (h.GetBinContent(bin+1)/h.GetBinWidth(bin+1)))

sigma_CtG_0_nlo = 687.14
sigma_CtG_neg2_nlo = 404.48
sigma_CtG_pos2_nlo = 1270.204
acceptance = 0.28371
branching_ratio = 0.046
k_factor = 1.21

fid_xsec = sigma_CtG_pos2_nlo * k_factor * branching_ratio * acceptance
integral = h.Integral()
h.Scale(fid_xsec / integral)

print "[",
for bin in range(0, h.GetNbinsX()):
    print str(h.GetBinContent(bin + 1) / (h.GetBinWidth(bin + 1))) + ",",
print "]"

f2 = TFile(
    "/Users/keaveney/EFT_Fitter/EFT_Fitter/files/Nov1/particle/absolute/results/DiffXS_HypLLBarDPhi_source.root"
)

g = f2.Get("data")
x, y = ROOT.Double(0), ROOT.Double(0)

print "[",
for bin in range(0, h.GetNbinsX()):
    g.GetPoint(bin, x, y)
    print str(y) + ",",
print "]"

f3 = TFile(
    "/Users/keaveney/EFT_Fitter/EFT_Fitter/files/Nov1/particle/absolute/covariance/HypLLBarDPhi_totCovEnvXSMtrxFile.root"
)
cov = f3.Get("cov")

print "cov matrix  = "
print "  "
print "[",
for bini in range(0, cov.GetNbinsX()):
    print "[",
    for binj in range(0, cov.GetNbinsY()):
        cov_ij = cov.GetBinContent(bini + 1, binj + 1)
        print str(cov_ij) + ",",
    print "]"
print "]"



# script to generate suitable sets of coefficient values
# for predictions for an observable in an EFT
import numpy as np
import random
import sys
from predBuilder import predBuilder

# ops = ["4", "5", "6", "8", "10"]

# 10 = ctW, 16 = ctG R, 6 = cPhit, 8 = cPhitB, 4 = cPhiQM, 5 = cPhiQt

# ops = ["6", "8", "10"]

# ops = ["4", "5", "6", "8", "10", "16"]

# ops = ["22", "23", "24"]

# cHWB:3 cW,:6, cHB:10, cHt:15, ctZ:22, ctW:23, ctG:24,

nTasks = 5
nSamples = 150

# ops = ["3", "6", "10", "15", "22", "23", "24"]

# SMEFTatNLO list
ops = ["12", "13", "15", "22", "23", "24"]

nOps = len(ops)

pb = predBuilder()


def genRandomCoeffSets(nOps):
    boostFact = 3.0
    # nSamples = 2*pb.nSamples(nOps)
    coeffSets = []

    for s in range(0, int(nSamples)):
        coeffSet = []
        coeffSet.append(1.0)

        for c in range(0, nOps):
            signs = [-1.0, 1.0]
            coeffSign = random.choice(signs)
            coeffMag = float(random.uniform(0, 30))
            randCoeff = coeffSign * coeffMag
            coeffSet.append(randCoeff)

        coeffSets.append(coeffSet)
    # coeffSets = coeffSets.reshape(int(nSamples), int(nSamples))
    return coeffSets


def genRandomPreds(nOps):

    nSamples = pb.nSamples(nOps)
    # preds = np.array([])
    preds = []

    for s in range(0, int(nSamples)):
        # pred = np.array([])
        pred = []

        # can make loop here for binned predictions
        # pred = np.append(pred, random.uniform(-100.0,100.0))
        # preds = np.append(preds, pred)
        pred.append(random.uniform(-50.0, 50.0))
        preds.append(pred)

    # np.reshape(preds, (int(nSamples),1))
    return preds


def writeProcCard(randCoeffSet):

    genFileName = "TWZ_6ops_gen.txt"
    genFile = open(genFileName, "w")

    # genFile.write("import dim6top_LO_UFO")
    genFile.write("import SMEFTatNLO-NLO")
    genFile.write("define p = p b b~")
    genFile.write("define tp = t t~")
    genFile.write("define w = w+ w-")
    genFile.write("define l+ = e+ mu+")
    genFile.write("define l- = e- mu-")
    genFile.write("define vl = ve vm")
    genFile.write("define vl~ = ve~ vm~")
    genFile.write("define lept = l+ l- vl vl~")
    genFile.write("generate p p > tp w z QCD=1 QED=2 NP=1 [QCD]")
    # genFile.write("generate p p > tp w z FCNC=0 DIM6=1")
    # genFile.write("generate p p > t t~ FCNC=0 DIM6=1, (t > w+ b DIM6=0, w+ > lept lept DIM6=0),(t~ > w- b~ DIM6=0, w- > lept lept DIM6=0)")
    # genFile.write("output /tmp/twz_train_7ops/")
    genFile.write("output /scratch/james/twz_train_6ops/")
    genFile.close()

    randCoeffSetAr = np.array(randCoeffSet)
    print("SHAPE = " + str(randCoeffSetAr.shape))

    # randCoeffSetAr = np.reshape(randCoeffSetAr, (nTasks,-1))
    print("SHAPE = " + str(randCoeffSetAr.shape[0]))

    pointsFileName = "pointsFile.txt"
    np.savetxt(pointsFileName, randCoeffSetAr, delimiter=",", newline="\n")

    if (nSamples % nTasks) != 0:
        sys.exit("nPoints must be an integer multiple of nTasks... exiting")

    for task in range(0, nTasks):
        trainFileName = "TWZ_6ops_train_" + str(task) + ".txt"
        # trainFileName = "TZQ_7ops_train_" + str(task) + ".txt"
        trainFile = open(trainFileName, "w")

        nSamplesPerTask = int(nSamples / nTasks)
        print("samples per task  = " + str(nSamplesPerTask))

        for point in range(0, nSamplesPerTask):
            trainFile.write("launch /scratch/james/twz_train_6ops_lo/\n")
            # trainFile.write("launch /scratch/james/tzq_train/\n")
            trainFile.write("order=LO\n")
            trainFile.write("fixed_order=ON\n")

            pointIndex = (task * ((nSamples / nTasks))) + point
            print("pointIndex = " + str(int(pointIndex)))
            for c in range(1, len(randCoeffSet[int(pointIndex)])):
                trainFile.write(
                    "set DIM62F "
                    + str(ops[c - 1])
                    + " "
                    + str(randCoeffSet[int(pointIndex)][c])
                    + "\n"
                )
        trainFile.close()

    for sample in range(0, len(randCoeffSet)):
        # print("launch /tmp/twz_train_7ops/")
        print("launch /scratch/james/twz_train_6ops_lo/")
        # print("launch /scratch/james/tzq_train/")
        print("order=LO")
        print("fixed_order=ON")
        # if sample ==0:
        # print("order=LO")
        # print("fixed_order=ON")
        # print("madspin=OFF")
        # print("shower=OFF")
        # print("reweight=OFF")
        # print("set nevents=5000")
        for c in range(1, len(randCoeffSet[sample])):
            print("set DIM62F " + str(ops[c - 1]) + " " + str(randCoeffSet[sample][c]))


randCoeffSet = genRandomCoeffSets(nOps)
randPreds = genRandomPreds(nOps)

print("random coeffs = " + str(randCoeffSet) + " random preds = " + str(randPreds))

# pb.init(nOps,randCoeffSet, randPreds)

writeProcCard(randCoeffSet)
